/*******************************************
 PDF Chain
 
 Copyright: 2009 Martin Singer
 <http://sourceforge.net/projects/pdfchain/>
 
 Author: Martin Singer
 <m_power3@users.sourceforge.net>
 
 License: GPL version 3+
 <http://www.gnu.org/licenses/>
 
 Tools
 src/fill_form.cc
 *******************************************/

#include "fill_form.hh"

/*** Constructor ***/
class_window_tools_fill_form::class_window_tools_fill_form() :
vbox_window(false, 5),
hbox_sourcefile(false, 5),
hbox_fdffile(false, 5),
hbox_dropxfa(false, 5),
hbox_flatten(false, 5),
frame_sourcefile(Text::str_SOURCE_FILE),
frame_fdffile(Text::str_FDF_FILE),
frame_dropxfa(Text::str_SELECT),
frame_flatten(Text::str_SELECT),
button_sourcefile(Gtk::Stock::ADD),
button_fdffile(Gtk::Stock::ADD),
button_save(Gtk::Stock::SAVE),
radiobutton_dropxfa(Text::str_DROP_XFA, false),
radiobutton_notdropxfa(Text::str_DO_NOT_DROP_XFA, false),
radiobutton_flatten(Text::str_FLATTEN, false),
radiobutton_notflatten(Text::str_DO_NOT_FLATTEN, false),
alignment_dropxfa(Gtk::ALIGN_LEFT, Gtk::ALIGN_CENTER, 0.0, 0.0),
alignment_flatten(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, 0.0, 0.0),
alignment_save(Gtk::ALIGN_CENTER, Gtk::ALIGN_CENTER, 0.0, 0.0){
	
	// Signal Handlers
	button_sourcefile.signal_clicked().connect(sigc::mem_fun(*this, &class_window_tools_fill_form::on_button_sourcefile));
	button_fdffile.signal_clicked().connect(sigc::mem_fun(*this, &class_window_tools_fill_form::on_button_fdffile));
	button_save.signal_clicked().connect(sigc::mem_fun(*this, &class_window_tools_fill_form::on_button_save));
	
	// Window
	set_title(Text::str_FILL_FORM);
	set_default_size(600, -1);
	set_border_width(0);
	set_position(Gtk::WIN_POS_CENTER);
	//set_resizable(false);
	//set_keep_above(true); 
	
	// Widgets
	button_sourcefile.set_size_request(150,-1);
	button_fdffile.set_size_request(150,-1);
	button_save.set_size_request(150,-1);
	entry_sourcefile.set_editable(false);
	entry_fdffile.set_editable(false);
	
	radiobutton_group_flatten = radiobutton_notflatten.get_group();
	radiobutton_flatten.set_group(radiobutton_group_flatten);
	radiobutton_notflatten.set_active();
	
	radiobutton_group_dropxfa = radiobutton_notdropxfa.get_group();
	radiobutton_dropxfa.set_group(radiobutton_group_dropxfa);
	radiobutton_notdropxfa.set_active();
	
	hbox_sourcefile.set_border_width(5);
	hbox_sourcefile.pack_start(entry_sourcefile,  Gtk::PACK_EXPAND_WIDGET, 0);
	hbox_sourcefile.pack_start(button_sourcefile, Gtk::PACK_SHRINK      , 0);
	frame_sourcefile.add(hbox_sourcefile);
	
	hbox_fdffile.set_border_width(5);
	hbox_fdffile.pack_start(entry_fdffile,  Gtk::PACK_EXPAND_WIDGET, 0);
	hbox_fdffile.pack_start(button_fdffile, Gtk::PACK_SHRINK      , 0);
	frame_fdffile.add(hbox_fdffile);
	
	hbox_dropxfa.set_border_width(5);
	hbox_dropxfa.pack_start(radiobutton_notdropxfa, Gtk::PACK_SHRINK, 0);
	hbox_dropxfa.pack_start(radiobutton_dropxfa,    Gtk::PACK_SHRINK, 0);
	frame_dropxfa.add(hbox_dropxfa);
	alignment_dropxfa.add(frame_dropxfa);
	
	hbox_flatten.set_border_width(5);
	hbox_flatten.pack_start(radiobutton_notflatten, Gtk::PACK_SHRINK, 0);
	hbox_flatten.pack_start(radiobutton_flatten,    Gtk::PACK_SHRINK, 0);
	frame_flatten.add(hbox_flatten);
	alignment_flatten.add(frame_flatten);
	
	hbox_selection.pack_start(alignment_dropxfa, Gtk::PACK_EXPAND_WIDGET);
	hbox_selection.pack_start(alignment_flatten, Gtk::PACK_EXPAND_WIDGET);
	
	hbox_save.pack_start(button_save, Gtk::PACK_SHRINK, 0);
	alignment_save.add(hbox_save);
	
	vbox_window.set_border_width(5);
	vbox_window.pack_start(frame_sourcefile,    Gtk::PACK_SHRINK, 0);
	vbox_window.pack_start(frame_fdffile,       Gtk::PACK_SHRINK, 0);
	vbox_window.pack_start(hbox_selection,      Gtk::PACK_SHRINK, 0);
	vbox_window.pack_start(alignment_save,      Gtk::PACK_SHRINK, 0);
	
	add(vbox_window);
	show_all_children();
	
	return;
}



/*** On Button -> Sourcefile ***/
void class_window_tools_fill_form::on_button_sourcefile(){
	
	// Create Filechooser
	class_filechoosers filechooser(this, Pdfchain::FILE_CHOOSER_OPEN, Pdfchain::FILE_FILTER_PDF);
	Glib::ustring filename = filechooser.run();
	
	if(filename != ""){
		
		entry_sourcefile.set_text(filename);
	}
	
	return;
}



/*** On Button -> FDF File ***/
void class_window_tools_fill_form::on_button_fdffile(){
	
	// Create Filechooser
	class_filechoosers filechooser(this, Pdfchain::FILE_CHOOSER_OPEN, Pdfchain::FILE_FILTER_FDF, Pdfchain::FILE_FILTER_TXT);
	Glib::ustring filename = filechooser.run();
	
	if(filename != ""){
		
		entry_fdffile.set_text(filename);
	}
	
	return;
}



/*** On Button Save ***/
void class_window_tools_fill_form::on_button_save(){
	
	// Check Errors -> No Source File
	if(entry_sourcefile.get_text() == ""){
		
		// Error Messages
		pt_dialogs->error_cerr("class_window_tools_fill_form::on_button_save()", Text::err_NO_SOURCE_FILE, Text::err_msg_EXIT_FUNCTION);
		pt_dialogs->error_message(this, Text::str_FILL_FORM, Text::err_NO_SOURCE_FILE);
		
		return;
	}
	
	// Check Errors -> No FDF File
	if(entry_fdffile.get_text() == ""){
		
		// Error Messages
		pt_dialogs->error_cerr("class_window_tools_fill_form::on_button_save()", Text::err_NO_FDF_FILE, Text::err_msg_EXIT_FUNCTION);
		pt_dialogs->error_message(this, Text::str_FILL_FORM, Text::err_NO_FDF_FILE);
		
		return;
	}
	
	Glib::ustring command;
	
	// Create Filechooser
	class_filechoosers filechooser(this, Pdfchain::FILE_CHOOSER_SAVE, Pdfchain::FILE_FILTER_PDF);
	Glib::ustring filename = filechooser.run();
	
	if(filename != ""){
		
		// Create Command
		command += Command::cmd_PDFTK + Command::cmd_SPACE;
		command += quote_out(entry_sourcefile.get_text());
		command += Command::cmd_FILL_FORM + Command::cmd_SPACE;
		command += quote_out(entry_fdffile.get_text());
		command += Command::cmd_OUTPUT + Command::cmd_SPACE;
		command += quote_out(filename);
		
		if(radiobutton_flatten.property_active() == true)
			command += Command::cmd_FLATTEN;
		
		else if(radiobutton_notflatten.property_active() == true)
			command += Command::cmd_NONE;
		
		else{
			command += Command::cmd_NONE;
			
			// Error Message
			pt_dialogs->error_cerr("class_window_tools_fill_form::on_button_save()", Text::err_UNKNOWN_SELECTION_FLATTEN, Text::str_DO_NOT_FLATTEN);
		}
		
		if(radiobutton_dropxfa.property_active() == true)
			command += Command::cmd_DROP_XFA;
		
		else if(radiobutton_notdropxfa.property_active() == true)
			command += Command::cmd_NONE;
		
		else{
			command += Command::cmd_NONE;
			
			// Error Message
			pt_dialogs->error_cerr("class_window_tools_fill_form::on_button_save()", Text::err_UNKNOWN_SELECTION_DROP_XFA, Text::str_DO_NOT_DROP_XFA);
		}
		
		// Execute Command
		execute(this, command);
	}
	
	return;
}



/*** Set Dialog ***/
void class_window_tools_fill_form::set_dialogs(class_dialogs* dialogs){
	
	pt_dialogs = dialogs;
	return;
}



/*** Clear ***/
void class_window_tools_fill_form::clear(){
	
	hide();
	entry_sourcefile.set_text("");
	entry_fdffile.set_text("");
	radiobutton_notdropxfa.set_active();
	radiobutton_notflatten.set_active();
	
	return;
}
