/*******************************************
 PDF Chain
 
 Copyright: 2009 Martin Singer
 <http://sourceforge.net/projects/pdfchain/>
 
 Author: Martin Singer
 <m_power3@users.sourceforge.net>
 
 License: GPL version 3+
 <http://www.gnu.org/licenses/>
 
 Tools
 src/update_info.cc
 *******************************************/

#include "update_info.hh"

/*** Constructor ***/
class_window_tools_update_info::class_window_tools_update_info() :
vbox_window(false, 5),
hbox_sourcefile(false, 5),
hbox_dumpfile(false, 5),
frame_sourcefile(Text::str_SOURCE_FILE),
frame_dumpfile(Text::str_DUMP_DATA_FILE),
button_sourcefile(Gtk::Stock::ADD),
button_dumpfile(Gtk::Stock::ADD),
button_save(Gtk::Stock::SAVE),
alignment_save(Gtk::ALIGN_CENTER, Gtk::ALIGN_CENTER, 0.0, 0.0){
	
	// Signal Handlers
	button_sourcefile.signal_clicked().connect(sigc::mem_fun(*this, &class_window_tools_update_info::on_button_sourcefile));
	button_dumpfile.signal_clicked().connect(sigc::mem_fun(*this, &class_window_tools_update_info::on_button_dumpfile));
	button_save.signal_clicked().connect(sigc::mem_fun(*this, &class_window_tools_update_info::on_button_save));
	
	// Window
	set_title(Text::str_UPDATE_INFO);
	set_default_size(600, -1);
	set_border_width(0);
	set_position(Gtk::WIN_POS_CENTER);
	//set_resizable(false);
	//set_keep_above(true); 
	
	// Widgets
	button_sourcefile.set_size_request(150,-1);
	button_dumpfile.set_size_request(150,-1);
	button_save.set_size_request(150,-1);
	entry_sourcefile.set_editable(false);
	entry_dumpfile.set_editable(false);
	
	hbox_sourcefile.set_border_width(5);
	hbox_sourcefile.pack_start(entry_sourcefile,  Gtk::PACK_EXPAND_WIDGET, 0);
	hbox_sourcefile.pack_start(button_sourcefile, Gtk::PACK_SHRINK      , 0);
	frame_sourcefile.add(hbox_sourcefile);
	
	hbox_dumpfile.set_border_width(5);
	hbox_dumpfile.pack_start(entry_dumpfile,  Gtk::PACK_EXPAND_WIDGET, 0);
	hbox_dumpfile.pack_start(button_dumpfile, Gtk::PACK_SHRINK      , 0);
	frame_dumpfile.add(hbox_dumpfile);
	
	hbox_save.pack_start(button_save, Gtk::PACK_SHRINK, 0);
	alignment_save.add(hbox_save);
	
	vbox_window.set_border_width(5);
	vbox_window.pack_start(frame_sourcefile,    Gtk::PACK_SHRINK, 0);
	vbox_window.pack_start(frame_dumpfile,       Gtk::PACK_SHRINK, 0);
	vbox_window.pack_start(alignment_save,      Gtk::PACK_SHRINK, 0);
	
	add(vbox_window);
	show_all_children();
	
	return;
}



/*** On Button -> Sourcefile ***/
void class_window_tools_update_info::on_button_sourcefile(){
	
	// Create Filechooser
	class_filechoosers filechooser(this, Pdfchain::FILE_CHOOSER_OPEN, Pdfchain::FILE_FILTER_PDF);
	Glib::ustring filename = filechooser.run();
	
	if(filename != ""){
		
		entry_sourcefile.set_text(filename);
	}
	
	return;
}



/*** On Button -> Dump File ***/
void class_window_tools_update_info::on_button_dumpfile(){
	
	// Create Filechooser
	class_filechoosers filechooser(this, Pdfchain::FILE_CHOOSER_OPEN, Pdfchain::FILE_FILTER_DUMP, Pdfchain::FILE_FILTER_TXT);
	Glib::ustring filename = filechooser.run();
	
	if(filename != ""){
		
		entry_dumpfile.set_text(filename);
	}
	
	return;
}



/*** On Button Save ***/
void class_window_tools_update_info::on_button_save(){
	
	// Check Errors -> No Source File
	if(entry_sourcefile.get_text() == ""){
		
		// Error Messages
		pt_dialogs->error_cerr("class_window_tools_update_info::on_button_save()", Text::err_NO_SOURCE_FILE, Text::err_msg_EXIT_FUNCTION);
		pt_dialogs->error_message(this, Text::str_UPDATE_INFO, Text::err_NO_SOURCE_FILE);
		
		return;
	}
	
	// Check Errors -> No Dump Data File
	if(entry_dumpfile.get_text() == ""){
		
		// Error Messages
		pt_dialogs->error_cerr("class_window_tools_update_info::on_button_save()", Text::err_NO_DUMP_DATA_FILE, Text::err_msg_EXIT_FUNCTION);
		pt_dialogs->error_message(this, Text::str_UPDATE_INFO, Text::err_NO_DUMP_DATA_FILE);
		
		return;
	}
	
	Glib::ustring command;
	
	// Create Filechooser
	class_filechoosers filechooser(this, Pdfchain::FILE_CHOOSER_SAVE, Pdfchain::FILE_FILTER_PDF);
	Glib::ustring filename = filechooser.run();
	
	if(filename != ""){
		
		// Create Command
		command += Command::cmd_PDFTK + Command::cmd_SPACE;
		command += quote_out(entry_sourcefile.get_text());
		command += Command::cmd_UPDATE_INFO + Command::cmd_SPACE;
		command += quote_out(entry_dumpfile.get_text());
		command += Command::cmd_OUTPUT + Command::cmd_SPACE;
		command += quote_out(filename);
		
		// Execute Command
		execute(this, command);
	}
	
	return;
}



/*** Set Dialog ***/
void class_window_tools_update_info::set_dialogs(class_dialogs* dialogs){
	
	pt_dialogs = dialogs;
	return;
}



/*** Clear ***/
void class_window_tools_update_info::clear(){
	
	hide();
	entry_sourcefile.set_text("");
	entry_dumpfile.set_text("");
	
	return;
}
